# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'settingsui.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SettingsUI(object):
    def setupUi(self, SettingsUI):
        SettingsUI.setObjectName("SettingsUI")
        SettingsUI.resize(763, 527)
        self.gridLayout = QtWidgets.QGridLayout(SettingsUI)
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox = QtWidgets.QGroupBox(SettingsUI)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.autoSaveCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.autoSaveCheckBox.setChecked(True)
        self.autoSaveCheckBox.setObjectName("autoSaveCheckBox")
        self.gridLayout_2.addWidget(self.autoSaveCheckBox, 0, 1, 1, 1)
        self.autoArchiveCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.autoArchiveCheckBox.setChecked(True)
        self.autoArchiveCheckBox.setObjectName("autoArchiveCheckBox")
        self.gridLayout_2.addWidget(self.autoArchiveCheckBox, 1, 1, 1, 1)
        self.addCreatedDateCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.addCreatedDateCheckBox.setChecked(True)
        self.addCreatedDateCheckBox.setObjectName("addCreatedDateCheckBox")
        self.gridLayout_2.addWidget(self.addCreatedDateCheckBox, 2, 1, 1, 1)
        self.confirmCompletionCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.confirmCompletionCheckBox.setChecked(True)
        self.confirmCompletionCheckBox.setObjectName("confirmCompletionCheckBox")
        self.gridLayout_2.addWidget(self.confirmCompletionCheckBox, 3, 1, 1, 2)
        self.deleteActionCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.deleteActionCheckBox.setObjectName("deleteActionCheckBox")
        self.gridLayout_2.addWidget(self.deleteActionCheckBox, 4, 1, 1, 2)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 5, 1, 1, 1)
        self.lowestPriorityLineEdit = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lowestPriorityLineEdit.sizePolicy().hasHeightForWidth())
        self.lowestPriorityLineEdit.setSizePolicy(sizePolicy)
        self.lowestPriorityLineEdit.setMaximumSize(QtCore.QSize(16777212, 16777215))
        self.lowestPriorityLineEdit.setObjectName("lowestPriorityLineEdit")
        self.gridLayout_2.addWidget(self.lowestPriorityLineEdit, 5, 2, 1, 1)
        self.trayCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.trayCheckBox.setObjectName("trayCheckBox")
        self.gridLayout_2.addWidget(self.trayCheckBox, 9, 1, 1, 2)
        self.frame = QtWidgets.QFrame(self.groupBox)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout.setContentsMargins(11, 0, 11, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.hideToTrayCheckBox = QtWidgets.QCheckBox(self.frame)
        self.hideToTrayCheckBox.setObjectName("hideToTrayCheckBox")
        self.verticalLayout.addWidget(self.hideToTrayCheckBox)
        self.hideOnStartupCheckBox = QtWidgets.QCheckBox(self.frame)
        self.hideOnStartupCheckBox.setObjectName("hideOnStartupCheckBox")
        self.verticalLayout.addWidget(self.hideOnStartupCheckBox)
        self.closeToTrayCheckBox = QtWidgets.QCheckBox(self.frame)
        self.closeToTrayCheckBox.setObjectName("closeToTrayCheckBox")
        self.verticalLayout.addWidget(self.closeToTrayCheckBox)
        self.gridLayout_2.addWidget(self.frame, 10, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 80, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 10, 4, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(261, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 11, 3, 1, 1)
        self.closeButton = QtWidgets.QPushButton(self.groupBox)
        self.closeButton.setObjectName("closeButton")
        self.gridLayout_2.addWidget(self.closeButton, 12, 4, 1, 1)
        self.singletonCheckBox = QtWidgets.QCheckBox(self.groupBox)
        self.singletonCheckBox.setObjectName("singletonCheckBox")
        self.gridLayout_2.addWidget(self.singletonCheckBox, 6, 1, 1, 1)
        self.colorSchemComboBox = QtWidgets.QComboBox(self.groupBox)
        self.colorSchemComboBox.setObjectName("colorSchemComboBox")
        self.colorSchemComboBox.addItem("")
        self.colorSchemComboBox.addItem("")
        self.colorSchemComboBox.addItem("")
        self.gridLayout_2.addWidget(self.colorSchemComboBox, 8, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 8, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)

        self.retranslateUi(SettingsUI)
        QtCore.QMetaObject.connectSlotsByName(SettingsUI)

    def retranslateUi(self, SettingsUI):
        _translate = QtCore.QCoreApplication.translate
        SettingsUI.setWindowTitle(_translate("SettingsUI", "SettingsUI"))
        self.groupBox.setTitle(_translate("SettingsUI", "QTodoTxt Settings"))
        self.autoSaveCheckBox.setText(_translate("SettingsUI", "Autosave"))
        self.autoArchiveCheckBox.setText(_translate("SettingsUI", "AutoArchive"))
        self.addCreatedDateCheckBox.setText(_translate("SettingsUI", "Add created date"))
        self.confirmCompletionCheckBox.setText(_translate("SettingsUI", "Ask for confirmation before task completion"))
        self.deleteActionCheckBox.setText(_translate("SettingsUI", "Show Delete action (Require restart)"))
        self.label.setText(_translate("SettingsUI", "Lowest task priority"))
        self.lowestPriorityLineEdit.setText(_translate("SettingsUI", "D"))
        self.trayCheckBox.setText(_translate("SettingsUI", "Enable system tray (Require restart)"))
        self.hideToTrayCheckBox.setText(_translate("SettingsUI", "Hide to tray"))
        self.hideOnStartupCheckBox.setText(_translate("SettingsUI", "Hide on startupp"))
        self.closeToTrayCheckBox.setText(_translate("SettingsUI", "CheckBox"))
        self.closeButton.setText(_translate("SettingsUI", "Close"))
        self.singletonCheckBox.setText(_translate("SettingsUI", "Singleton mode"))
        self.colorSchemComboBox.setItemText(0, _translate("SettingsUI", "System"))
        self.colorSchemComboBox.setItemText(1, _translate("SettingsUI", "dark_blue"))
        self.colorSchemComboBox.setItemText(2, _translate("SettingsUI", "dark_orange"))
        self.label_2.setText(_translate("SettingsUI", "Theme"))

